// IndustryPackIpac.cc

/********************COPYRIGHT NOTIFICATION**********************************
  This software was developed under a United States Government license
  described on the COPYRIGHT_UniversityOfChicago file included as part
  of this distribution.
  ***************************************************************************
  */
/*
  Author: Andrew Johnson
  Date: July 1999
  */

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include <vxLib.h>
#include <intLib.h>

#include "DLList.h"
#include "IndustryPackCarrier.h"
#include <drvIpac.h>


#define IPAC_MAX_SITES 4

#define IPAC_SLOTA_NAME "IP_a"
#define IPAC_SLOTB_NAME "IP_b"
#define IPAC_SLOTC_NAME "IP_c"
#define IPAC_SLOTD_NAME "IP_d"

//

class IndustryPackSiteIpac;


// Carrier class definition

class IndustryPackCarrierIpac : public IndustryPackCarrier {
public:
    IndustryPackCarrierIpac(const char *carrierName, unsigned short carrierNum);
    bool probeSite(unsigned short slot);
    const char* getSiteName(unsigned short slot);
    
protected:
    unsigned short carrier;

private:
    volatile void *allocMemMapIP(int size);
    
    friend class IndustryPackCarrierSiteIpac;
};


// Site class definition

class IndustryPackSiteIpac : public IndustryPackSite {
public:
    IndustryPackSiteIpac(IndustryPackCarrierIpac *pCarrier, 
    			 unsigned short carrierNum, unsigned short slotNum);

    // interrupt control functions valid num: 0=IRQ0, 1=IRQ1
    volatile void* allocMemMapIP(int size);
    volatile IP_ID_PROM* getMemBaseID();
    volatile void* getMemBaseIO();
    volatile void* getMemBaseIP();
    int intConfig(int num);
    void intEnable(int num);
    void intDisable(int num);
    void intClear(int num);

private:
    unsigned short carrier;
    unsigned short slot;

    friend class IndustryPackCarrierIpac;
};


// Helper function for use in the startup script

int initIpacCarrier(const char *carrierName, int carrierNum) {
    // make sure this carrier not already created
    IndustryPackCarrier *pIPC = IndustryPackCarrier::find(carrierName);
    if (pIPC) {
	printf("initIndustryPackIpac already initialized %s\n",carrierName);
	return(0);
    }
    new IndustryPackCarrierIpac(carrierName, (unsigned short) carrierNum);
    return (0);
}


// Carrier class: public member functions

IndustryPackCarrierIpac::IndustryPackCarrierIpac(const char *name, unsigned short carrierNum) {
    // printf("IndustryPackCarrierIpac::IndustryPackCarrierIpac name %s\n",name);
    registerName(name);
    carrier = carrierNum;

    for (int i=0; i<IPAC_MAX_SITES; i++) {
	// printf ("IndustryPackCarrierIpac::IndustryPackCarrierIpac probing site %d\n",i);
	bool probe = probeSite(i);
	// printf ("IndustryPackCarrierIpac::IndustryPackCarrierIpac probe result %d\n",probe);

	if (probe) {
	    IndustryPackSiteIpac *pSite = new IndustryPackSiteIpac(this, carrier, i);
	    addSite(pSite);
	}
    }

    return;
}

volatile void *IndustryPackCarrierIpac::allocMemMapIP(int size) {
    printf("!! IndustryPackCarrierIpac::allocMemMapIP(%d) shouldn't be being called!\n", size);
    return NULL;
}

// Carrier class: private member functions

const char *IndustryPackCarrierIpac::getSiteName(unsigned short slot) {
    switch (slot) {
    case 0:
	return IPAC_SLOTA_NAME;

    case 1:
	return IPAC_SLOTB_NAME;

    case 2:
	return IPAC_SLOTC_NAME;

    case 3:
	return IPAC_SLOTD_NAME;

    // In the event any carrier drivers are written for boards that support 
    // more than 4 slots this routine will need updating.

    default:
    	break;
    }
    return NULL;
}


bool IndustryPackCarrierIpac::probeSite(unsigned short slot) {
    return (ipmCheck(carrier, slot) == OK);
}


// Site class: public member functions

IndustryPackSiteIpac::IndustryPackSiteIpac (IndustryPackCarrierIpac *pCarrier, 
    unsigned short carrierNum, unsigned short slotNum) {
    pIpCarrier = pCarrier;
    carrier = carrierNum;
    slot = slotNum;
    siteName = pCarrier->getSiteName(slotNum);
    
    printf("IndustryPackSiteIpac:: carrier %s site %s initialized Ok.\n", 
	    pCarrier->getCarrierName(), getSiteName());
}


volatile void *IndustryPackSiteIpac::allocMemMapIP(int size) {
    // The memory should have been allocated already, just check it here
    volatile void *memBase = getMemBaseIP();
    if ((memBase == NULL) && (size > 0)) {
    	printf("IP Carrier %s site %s needs to be configured for %d bytes memory.\n",
		pIpCarrier->getCarrierName(), getSiteName(), size);
    }
    return memBase;
}


volatile IP_ID_PROM *IndustryPackSiteIpac::getMemBaseID() {
    return (IP_ID_PROM *) ipmBaseAddr(carrier, slot, ipac_addrID);
}


volatile void * IndustryPackSiteIpac::getMemBaseIO() {
    return ipmBaseAddr(carrier, slot, ipac_addrIO);
}


volatile void * IndustryPackSiteIpac::getMemBaseIP() {
    return ipmBaseAddr(carrier, slot, ipac_addrMem);
}


int IndustryPackSiteIpac::intConfig(int ) {
    return 0;
}


void IndustryPackSiteIpac::intEnable(int num) {
    ipmIrqCmd(carrier, slot, num, ipac_irqEnable);
    // Let's be friendly, tell the user if the interrupt level is zero
    int level = ipmIrqCmd(carrier, slot, num, ipac_irqGetLevel);
    if (level == 0) {
    	printf("IP Carrier %s site %s needs irq %d to be configured.\n",
		pIpCarrier->getCarrierName(), getSiteName(), num);
    }
}


void IndustryPackSiteIpac::intDisable(int num) {
    if (ipmIrqCmd(carrier, slot, num, ipac_irqDisable)) {
	printf("IndustryPackSiteIpac::intDisable not supported here.\n");
    }
}


void IndustryPackSiteIpac::intClear(int num) {
    if (ipmIrqCmd(carrier, slot, num, ipac_irqClear)) {
	printf("IndustryPackSiteIpac::intClear not supported here.\n");
    }
}
